"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _chalk = _interopRequireDefault(require("chalk"));

var _cliSpinners = _interopRequireDefault(require("cli-spinners"));

var _events = _interopRequireDefault(require("events"));

var _interface = _interopRequireDefault(require("@wdio/interface"));

var _logger = _interopRequireDefault(require("@wdio/logger"));

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const log = (0, _logger.default)('@wdio/cli');
const clockSpinner = _cliSpinners.default['clock'];
const MAX_RUNNING_JOBS_DISPLAY_COUNT = 10;

class WDIOCLInterface extends _events.default {
  constructor(config, specs, totalWorkerCnt) {
    super();
    this.hasAnsiSupport = !!_chalk.default.supportsColor.hasBasic;
    this.isTTY = !!process.stdout.isTTY;
    this.specs = specs;
    this.config = config;
    this.totalWorkerCnt = totalWorkerCnt;
    this.sigintTriggered = false;
    this.isWatchMode = false;
    this.interface = new _interface.default();
    this.interface.on('bufferchange', this.updateView.bind(this));
    this.on('job:start', this.addJob.bind(this));
    this.on('job:end', this.clearJob.bind(this));
    this.setup();
  }

  setup() {
    this.clockTimer = 0;
    this.jobs = new Map();
    this.start = Date.now(); // The relationship between totalWorkerCnt and these counters are as follows:
    //   totalWorkerCnt - retries = finished = passed + failed

    this.result = {
      finished: 0,
      passed: 0,
      retries: 0,
      failed: 0
    };
    this.messages = {
      /**
       * messages from worker reporters
       */
      reporter: {},

      /**
       * messages from worker itself
       */
      worker: {}
    };
    this.interface.clearBuffer();
  }
  /**
   * add job to interface
   */


  addJob({
    cid,
    caps,
    specs
  }) {
    this.jobs.set(cid, {
      caps,
      specs
    });
    this.updateView();
  }
  /**
   * clear job from interface
   */


  clearJob({
    cid,
    passed,
    retries
  }) {
    this.jobs.delete(cid);
    const retry = !passed && retries > 0;

    if (!retry) {
      this.result.finished++;
    }

    if (passed) {
      this.result.passed++;
    } else if (retry) {
      this.totalWorkerCnt++;
      this.result.retries++;
    } else {
      this.result.failed++;
    }

    this.updateView(true);
  }
  /**
   * event handler that is triggered when runner sends up events
   */


  onMessage(event) {
    if (event.origin === 'debugger' && event.name === 'start') {
      clearTimeout(this.interval);
      this.interface.clearAll();
      this.interface.inDebugMode = true;
      this.interface.log(_chalk.default.yellow(event.params.introMessage));
    }

    if (event.origin === 'debugger' && event.name === 'stop') {
      this.interface.inDebugMode = false;
      this.sigintTriggered = false;
      return this.updateView();
    }

    if (!event.origin || !this.messages[event.origin]) {
      return log.warn(`Can't identify message from worker: ${JSON.stringify(event)}, ignoring!`);
    }

    if (!this.messages[event.origin][event.name]) {
      this.messages[event.origin][event.name] = [];
    }

    this.messages[event.origin][event.name].push(event.content);
    this.updateView();
  }

  sigintTrigger() {
    /**
     * allow to exit repl mode via Ctrl+C
     */
    if (this.interface.inDebugMode) {
      return;
    }

    this.sigintTriggered = true;
    this.updateView();
  }

  updateView(wasJobCleared) {
    const totalJobs = this.totalWorkerCnt - this.result.retries;
    const pendingJobs = totalJobs - this.jobs.size - this.result.finished;
    const runningJobs = this.jobs.size;
    const isFinished = runningJobs === 0;

    if (this.sigintTriggered) {
      clearTimeout(this.interval);
      this.interface.log('\n');
      return this.interface.log(!isFinished ? 'Ending WebDriver sessions gracefully ...\n' + '(press ctrl+c again to hard kill the runner)' : 'Ended WebDriver sessions gracefully after a SIGINT signal was received!');
    }

    if (isFinished) {
      return;
    }
    /**
     * check if environment supports ansi or does not
     * support TTY and print a limited update if not
     */


    if (!this.hasAnsiSupport || !this.isTTY) {
      /**
       * only update if a job finishes
       */
      if (!wasJobCleared) {
        return;
      }

      const clockSpinnerSymbol = this.getClockSymbol();
      return this.interface.log(`${clockSpinnerSymbol} ` + `${this.jobs.size} running, ` + `${this.result.passed} passed, ` + (this.result.retries ? `${this.result.retries} retries, ` : '') + `${this.result.failed} failed, ` + `${totalJobs} total ` + `(${Math.round(this.result.finished / totalJobs * 100)}% completed)`);
    }

    this.interface.clearAll();
    this.interface.log();
    /**
     * print running jobs
     */

    for (const [cid, job] of Array.from(this.jobs.entries()).slice(0, MAX_RUNNING_JOBS_DISPLAY_COUNT)) {
      const filename = job.specs.join(', ').replace(process.cwd(), '');
      this.interface.log(_chalk.default.bgYellow.black(' RUNNING '), cid, 'in', (0, _utils.getRunnerName)(job.caps), '-', filename);
    }
    /**
     * show number of pending and running jobs
     */


    if (pendingJobs || runningJobs > MAX_RUNNING_JOBS_DISPLAY_COUNT) {
      const logString = [];

      if (runningJobs > MAX_RUNNING_JOBS_DISPLAY_COUNT) {
        logString.push(runningJobs - MAX_RUNNING_JOBS_DISPLAY_COUNT, 'running tests' + (pendingJobs ? ' -' : ''));
      }

      if (pendingJobs) {
        logString.push(pendingJobs, 'pending tests');
      }

      this.interface.log(_chalk.default.yellow('...', ...logString.filter(l => Boolean(l))));
    }
    /**
     * print reporters in watch mode
     */


    if (this.isWatchMode) {
      this.printReporters();
    }
    /**
     * add empty line between "pending tests" and results
     */


    if (this.jobs.size) {
      this.interface.log();
    }

    this.printStdout(5);
    this.printSummary();
    this.updateClock();
  }

  printReporters() {
    this.interface.log();
    /**
     * print reporter output
     */

    for (const [reporterName, messages] of Object.entries(this.messages.reporter)) {
      this.interface.log(_chalk.default.bgYellow.black(`"${reporterName}" Reporter:`));
      this.interface.log(messages.join(''));
      this.interface.log();
    }
  }
  /**
   * print stdout and stderr from runners
   */


  printStdout(length) {
    if (this.interface.stdoutBuffer.length) {
      const bufferLength = this.interface.stdoutBuffer.length;
      const maxBufferLength = !length || length > bufferLength ? bufferLength : length;
      const buffer = this.interface.stdoutBuffer.slice(bufferLength - maxBufferLength);
      this.interface.log(_chalk.default.bgYellow.black('Stdout:\n') + buffer.join(''));
    }

    if (this.interface.stderrBuffer.length) {
      const bufferLength = this.interface.stderrBuffer.length;
      const maxBufferLength = !length || length > bufferLength ? bufferLength : length;
      const buffer = this.interface.stderrBuffer.slice(bufferLength - maxBufferLength);
      this.interface.log(_chalk.default.bgRed.black('Stderr:\n') + buffer.join(''));
    }

    if (this.messages.worker.error) {
      const bufferLength = this.messages.worker.error.length;
      const maxBufferLength = !length || length > bufferLength ? bufferLength : length;
      const buffer = this.messages.worker.error.slice(bufferLength - maxBufferLength);
      this.interface.log(_chalk.default.bgRed.black('Worker Error:\n') + buffer.map(e => e.stack).join('\n') + '\n');
    }
  }

  printSummary() {
    const totalJobs = this.totalWorkerCnt - this.result.retries;
    return this.interface.log('Test Suites:\t', _chalk.default.green(this.result.passed, 'passed') + ', ' + (this.result.retries ? _chalk.default.yellow(this.result.retries, 'retries') + ', ' : '') + (this.result.failed ? _chalk.default.red(this.result.failed, 'failed') + ', ' : '') + totalJobs, 'total', `(${totalJobs ? Math.round(this.result.finished / totalJobs * 100) : 0}% completed)`);
  }

  updateClock(interval = 100) {
    const clockSpinnerSymbol = this.getClockSymbol();
    clearTimeout(this.interval);
    this.interface.clearLine();
    this.interface.write('Time:\t\t ' + clockSpinnerSymbol + ' ' + ((Date.now() - this.start) / 1000).toFixed(2) + 's');
    this.interval = setTimeout(() => this.updateClock(interval), interval);
  }

  getClockSymbol() {
    return clockSpinner.frames[this.clockTimer = ++this.clockTimer % clockSpinner.frames.length];
  }

  reset() {
    clearTimeout(this.interval);
    this.interface.log('\n');

    if (this.isWatchMode) {
      return;
    }

    this.interface.reset();
  }

  finalise() {
    this.interface.clearAll();
    this.printReporters();
    this.printStdout();
    this.printSummary();
    this.updateClock();
    this.reset();
  }

}

exports.default = WDIOCLInterface;